////////////////////////////////////////////////////////////////////////////////////
// TracerECM Advaced Voice Pack Support.
//
// Purpose: To allow a more dynamic system for adding custom sounds to
//          TracerDX Chat menu and system Chat Menu than existing modules.
//
// Author:  Bryan W. -aka-> TracerDX
//
////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////
// Release Notes:
// + Feature Addition  - Bug Fix   * Misc Change
//
//
// 1.0 * Original Release
//
// 1.1 + Added ECMVoicePacks.issueCommand data item so scripts can set what command
//       to issue on a bind. (ECM Uses this to bind VPDX Packs to its menu and
//       have it's own handlers
//     + Added parameter: %overrideK to addVPDXItem(). This is so nullvoice and other
//       more dynamic items may be added to voicepacks. (Needed this for the
//       Nullvoice VoicePack I released with ECM 4.0)
//     * Removed an echo I left in here for debugging on accident.
//     - (No bugs reported yet :) )
////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////
// ECMVoicePacks Initialization
////////////////////////////////////////////////////////////////////////////////////
if(isObject(ECMVoicePackDriver))
	ECMVoicePackDriver.delete();
	
new ScriptObject(ECMVoicePackDriver) {

	class = "ECMVoicePackDriver";

	VoicePackCount = 0;
	Version = "2.0";
	ConverterMode = false;
	OldPackCount = -1;
	About = "ECMVoicePacks is a support script designed allow Voice Pack authors more flexibilty in Voice Pack development and deployment. For more information, point your favorite browser to http://www.planettracerdx.net";

	ChatMenuFunction = "setChatMenu";
	CancelMenuFunction = "cancelChatMenu";
	issueCommand = "issueVPDXCmd";
};


////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Registration/Definition Entry Point Functions
////////////////////////////////////////////////////////////////////////////////////

// %PackName			-	Display Name of package (No String limitation)
// %PackFileObjName		-	ObjectName of package (File Name limitations apply and no spaces)
// %PackKey				-	Default entry point keys
// %PackAuthor			-	Author of pack for credits displat
// %PackDescription		-	Description of Pack for credits display
// %PackVersion			-	Version of this pack.
//function RegisterVoicePackage( %PackName, %PackFileObjName, %PackKeys, %PackAuthor, %PackDescription, %PackVersion ) {

	//Register all data members into our DB
//	ECMVoicePacks.PackDisplayName[ECMVoicePacks.PackCount] = %PackName;
//	ECMVoicePacks.PackObjName[ECMVoicePacks.PackCount] = %PackFileObjName;
//	ECMVoicePacks.PackAuthor[ECMVoicePacks.PackCount] = %PackAuthor;
//	ECMVoicePacks.PackDefaultKeys[ECMVoicePacks.PackCount] = %PackKeys;
//	ECMVoicePacks.PackDescription[ECMVoicePacks.PackCount] = %PackDescription;
//	ECMVoicePacks.PackVersion[ECMVoicePacks.PackCount] = %PackVersion;
	
	//Console Registration Message
//	echo( "\c4Voice Package #" @ ECMVoicePacks.PackCount @ ": " @ ECMVoicePacks.PackDisplayName[ECMVoicePacks.PackCount] @ " v" @ ECMVoicePacks.PackVersion[ECMVoicePacks.PackCount] @ " - [" @ ECMVoicePacks.PackObjName[ECMVoicePacks.PackCount] @ "] by " @ ECMVoicePacks.PackAuthor[ECMVoicePacks.PackCount] @ " has been Registered Successfully" );

	//Increment for next registration
//	ECMVoicePacks.PackCount++;

//}

////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Scan
////////////////////////////////////////////////////////////////////////////////////
// This nifty little function scans for voicepacks and installs them automatically
// No need for stupid registration scripts
////////////////////////////////////////////////////////////////////////////////////

function ECMVoicePackDriver::ScanForVoicePacks(%this) {

	echo("\c6ECM Voicepack Driver:\c3 Scanning for Voicepacks...");

	//Get a list of all script files in the voicepacks directory
	for(%file = findFirstFile("scripts/voicepacks/*.cs"); %file !$= ""; %file = findNextFile("scripts/voicepacks/*.cs"))
		%fileList[%fileCount++] = %file;
		
	%fileObj = new FileObject();

	//Scan All files for voicepack headers
	for (%i = 1; %i <= %fileCount; %i++) {
	
		%fileObj.openForRead(%fileList[%i]);
		
		//First line of voicepack definition file MUST be "#//VOICEPACK"
		if (%fileObj.readLine() $= "//#VOICEPACK") {
		
			%this.VoicePacks[%this.VoicePackCount] = new ScriptObject() {
			
				class = "ECMVoicePack";
				file = %fileList[%i];
				
			};
			
			%this.VoicePacks[%this.VoicePackCount].isCustom = false;
			
			//Loop while the begining of each line = "//#"
			while (getSubStr((%directiveLine = %fileObj.readLine()), 0, 3) $= "//#") {
			
				switch$ (getWord(%directiveLine, 0)) {
				
					case "//#NAME":
						%this.VoicePacks[%this.VoicePackCount].Name = getSubStr(%directiveLine, 8, (strlen(%directiveLine) - 8));
					case "//#ID":
						%this.VoicePacks[%this.VoicePackCount].ID = getSubStr(%directiveLine, 6, (strlen(%directiveLine) - 6));
					case "//#VERSION":
						%this.VoicePacks[%this.VoicePackCount].Version = getSubStr(%directiveLine, 11, (strlen(%directiveLine) - 11));
					case "//#AUTHOR":
						%this.VoicePacks[%this.VoicePackCount].Author = getSubStr(%directiveLine, 10, (strlen(%directiveLine) - 10));
					case "//#DESCRIPTION":
						%this.VoicePacks[%this.VoicePackCount].Description = getSubStr(%directiveLine, 15, (strlen(%directiveLine) - 15));
					case "//#DEFAULTKEY":
						%this.VoicePacks[%this.VoicePackCount].DefaultKey = getSubStr(%directiveLine, 14, (strlen(%directiveLine) - 14));
					case "//#ISCUSTOM":
						%this.VoicePacks[%this.VoicePackCount].isCustom = true;
				
				}
			
			}
			
			echo("\c3 - Found [#" @ %this.VoicePackCount @ "] - " @ %this.VoicePacks[%this.VoicePackCount].Name @ " v" @ %this.VoicePacks[%this.VoicePackCount].Version @ " by " @ %this.VoicePacks[%this.VoicePackCount].Author @ " - Key=" @ %this.VoicePacks[%this.VoicePackCount].DefaultKey);
			
			//Check to see if this pack has been loaded before, then set the flag appropriately
			%this.VoicePacks[%this.VoicePackCount].Initialized = (strpos($pref::TracerECM::InstalledVoicepacks, %this.VoicePacks[%i].ID) != -1);
			
			
			%this.VoicePackCount++;
		
		
		}
		
		%fileObj.close();
		
	}
	
	echo("\c6 --- Scan Complete ---");

}

////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Install
////////////////////////////////////////////////////////////////////////////////////
// This function installs all voicepacks found durring the scan and "executes" them 
// to create the menu objects for them. These menu objects are used by ECM when
// the Entry Point Key, defined by the #DEFAULTKEY directive, or by the user, is 
// pressed on ECM
////////////////////////////////////////////////////////////////////////////////////

function ECMVoicePackDriver::InstallVoicePacks(%this) {

	//Clear existing menu objects
	if (isObject(VoicePackCleanUpList))
		VoicePackCleanUpList.delete();
	
	new SimGroup(VoicePackCleanUpList);

	for (%i = 0; %i < %this.VoicePackCount; %i++) {
	
		echo("\c6ECM Voicepack Driver:\c3 Installing Voicepack #" @ %i @ " - " @ %this.VoicePacks[%i].Name);
	
		//Step 1: Create Root menu item for this voicepack
		%thisMenu = (%this.VoicePacks[%i].rootMenu = new ActionMap());
		VoicePackCleanUpList.add( %thisMenu );
		%thisMenu.bindCmd(keyboard, escape, "ECMShowHUD.cancelMenu();", "");
		%thisMenu.optionCount = 0;
		%this.CurrentVoicePack = %this.VoicePacks[%i];
		
		//Step 2: Set newly created menu object as the $CurrentVPDXMenu
		$CurrentVMenu = %thisMenu;
		
		//Step 3: Execute script to bind the menu structure to this object
		exec(%this.VoicePacks[%i].file);
		
		//Step 4: Bind all other keys to kill menu
		%thisMenu.bindCmd(keyboard, anyKey, "ECMShowHUD.cancelMenu();", "");
	
	}

}

////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Build Functions
////////////////////////////////////////////////////////////////////////////////////
// A voice pack definition file consists of these functions, called in sequence
// to construct a menu.
////////////////////////////////////////////////////////////////////////////////////

//Menu Build Functions - Start a new submenu
// %heading		- "X: Heading" where X = key to bind menu to in current level.
function startVMenu(%heading)
{
	%key = getSubStr( %heading, 0, strpos(%heading, ":") );
	%text = getSubStr(%heading, (strpos(%heading, ":")+2), (strlen(%heading) - (strpos(%heading, "-")+2)));

	%menu = new ActionMap();
	VoicePackCleanUpList.add( %menu );
	%cm = $CurrentVMenu;
	%cm.bindCmd(keyboard, %key, "ECMShowHUD.setMenu(\"" @ %text @ "\", " @ %menu @ ");", "");
	%cm.option[%cm.optionCount] = %key @ ": " @ %text;
	%cm.command[%cm.optionCount] = %menu;  // Save this off here for later...
	%cm.isMenu[%cm.optionCount] = 1;
	%cm.isVoicePack[%cm.optionCount] = true;		//This is a little value we use to figure out if this bind was added by a voicepack or not
	%cm.optionCount++;
	%menu.parent = %cm;
	%menu.bindCmd(keyboard, escape, "ECMShowHUD.cancelMenu();", "");
	%menu.optionCount = 0;
	$CurrentVMenu = %menu;
}

//Menu Build Functions - end current menu
function endVMenu() {
	$CurrentVMenu = $CurrentVMenu.parent;
}

//Item Build Function
// %keyDesc		- "X: Item Lable" where X = key to bind item to in current level
// %text		- Text to say for this bind
// %sound		- Sound to play for this bind
// %isTeamChat	- true/false for team chat item
function addVItem(%keyDesc, %text, %sound, %isTeamChat)
{

	//SoundExplorer.src.cs - This registers a "pretty" label for this wav file to be displayed in the sound explorer
	//Holy shit this slows shit down... find a better way!
	//ECMSoundExplorer.AddVPWavResourceLabel("voice/custom/" @ %sound @ ".wav", %text);

	//Parse Data Items from %heading string
	%key = getSubStr( %keyDesc, 0, strpos(%keyDesc, ":") );
	%menutext = getSubStr(%keyDesc, (strpos(%keyDesc, ":")+2), (strlen(%keyDesc) - (strpos(%keyDesc, ":")+2)));
	
	//Setup an ECM Command Object for this
	%commandObj = new scriptObject() {
		VPchatText = %text;
		VPchatSound = %sound;
		isVoicePackItem = true;
		isTeam = %isTeamChat;
	};
	VoicePackCleanUpList.add( %commandObj );
	
	//From Previous menu build, grab the current menu stored in global $CurrentVPDXMenu
	%currMenu = $CurrentVMenu;

	//Add a keybind for this chat command
	//+1.1: issueVPDXCmd no longer hard coded. This way 3rd party scripters can have VPDX use a diff handler function
	%currMenu.bindCmd(keyboard, %key, issueECMCmd @ "(" @ %currMenu @ ", " @ %currMenu.optionCount @ ");", "");
	%currMenu.option[%currMenu.optionCount] = %keyDesc;
	%currMenu.command[%currMenu.optionCount] = %commandObj;
	%currMenu.isMenu[%currMenu.optionCount] = 0;
    	%currMenu.isVoicePack[%currMenu.optionCount] = true;		//This is a little value we use to figure out if this bind was added by a voicepack or not

	%currMenu.optionCount++;

}


///////////////////////////////////////////////////////////////////////////////////
// GUI Handler Functions
///////////////////////////////////////////////////////////////////////////////////

//Main Setup GUI launched
function ECMVoicePacksManager::onWake( %this ) {

	ECMVPPackList.fillList();

}


//Voice Pack listing needs to be (re)filled
function ECMVPPackList::fillList( %this ) {

	%this.clear();
	for ( %i = 0; %i < ECMVoicePackDriver.VoicePackCount; %i++ ) {

		%this.addRow( %i, ("\c5" @ ECMVoicePackDriver.VoicePacks[%i].Name @ " v" @ ECMVoicePackDriver.VoicePacks[%i].Version @ " by " @ ECMVoicePackDriver.VoicePacks[%i].Author ) );

	}

}


//Info Page GUI Launched
function ECMVoicePacksInfoPage::onWake( %this ) {
	
	%InfoPageNum = VPDXPackList.getSelectedId();
	//$VPDXInfoPageNum = %InfoPageNum;
	VPDXGUIPackName.setText(ECMVoicePackDriver.VoicePacks[%InfoPageNum].Name);
	VPDXGUIAuthor.setText(ECMVoicePackDriver.VoicePacks[%InfoPageNum].Author);
	VPDXGUIVersion.setText(ECMVoicePackDriver.VoicePacks[%InfoPageNum].Version);
	VPDXGUIDescription.setText(ECMVoicePackDriver.VoicePacks[%InfoPageNum].Description);

	//%keys = ECMVoicePacks.PackKeys[%InfoPageNum];
	//if (%keys $= "")
	//	%keys = ECMVoicePacks.PackDefaultKeys[%InfoPageNum];

	//VPDXPackKeys.setText(%keys);

	//if (ECMVoicePacks.PackProblem[%InfoPageNum] == 1)
	//	VPDXGUIProblem.setText("Tried to bind to non-existent bind path. Choose another above.");
	//else if (ECMVoicePacks.PackProblem[%InfoPageNum] == 2)
	//	VPDXGUIProblem.setText("Tried to bind menu to existing command. Choose another above.");
	//else if (ECMVoicePacks.PackProblem[%InfoPageNum] == 3)
	//	VPDXGUIProblem.setText("Bind Problem: Key Bind Sequence in use! Choose another above.");
	//else
	//	VPDXGUIProblem.setText("");

}

//function VPDXUpdatePackKeys() {

//	ECMVoicePacks.PackKeys[$VPDXInfoPageNum] = $VPDXKeys;

//	VPDXSaveKeys();

//	RebuildVPDXMenus();
//	VPDXPackList.fillList();
//	Canvas.popDialog( ECMVoicePacksInfoPage );

//}

////////////////////////////////////////////////////////////////////////////////////
// Over-ride functions
////////////////////////////////////////////////////////////////////////////////////
//deactivatePackage(ECMVoicePacks);
package ECMVoicePacks {


	//This function intercepts incoming chat bind messages, we will need to grap voicepack messages here.
	function clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
	{
		%message = detag( %msgString ); $MaxMessageWavLength = 120000; //120000 = 2 minutes
		if( (%message $= "" ) || isClientChatMuted(%sender) )
			return;

		//Ignore voicepacks if applicable
		if ($pref::TracerECM::IgnoreVoicePacks && (strpos(%message, "~wK") != -1))
			return;		

		//Mute Nullvoice binds if applicable
		if ($pref::TracerECM::MuteNullvoice && (detag(%voice) $= "") && (strpos(%message, "~wK") == -1))
			%message = getSubStr(%message, 0, strpos(%message, "~w"));

		//Subdue VoicePacks
		if ($pref::TracerECM::SubduedVoicePacks && (strpos(%message, "~wK") != -1))
			%message = getSubStr(%message, 0, strpos(%message, "\co")) @ TracerECM.VoicePackColor @ getSubStr(%message, (strpos(%message, "\co") + 1), strlen(%message));

		%wavStart = strstr(%message,"~w");
		if ( %wavStart == -1 )
			addMessageHudLine(%message);
		else
			if(strstr(%message, "~wK") !=-1) {
				%voice = "custom"; //Assign our base directory here.
				%wav = getSubStr(%message, %wavStart + 3, 1000); //Move the name three spaces for our delim
				%wavFile = "voice/" @ %voice @ "/" @ %wav @ ".wav";
				if (%pitch < 0.5 || %pitch > 2.0)
					%pitch = 1.0;
				%wavLengthMS = alxGetWaveLen(%wavFile) * %pitch;
				if ( $ClientChatHandle[%sender] != 0 )
					alxStop( $ClientChatHandle[%sender] );
				$ClientChatHandle[%sender] = alxCreateSource( AudioChat, %wavFile );
				if (%pitch != 1.0)
					alxSourcef($ClientChatHandle[%sender], "AL_PITCH", %pitch);
				alxPlay( $ClientChatHandle[%sender] );
				%message = getSubStr(%message, 0, %wavStart);
				addMessageHudLine(%message);
			} else
				parent::clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
				// Call back using the parent process to handle it if we don't have custom sound.
	}

};
activatePackage(ECMVoicePacks);


//This function must be called at Dispatch launch mode to install voicepacks at "defaultkey" entries.
//It will call the ECM gui and some popups, so it needs to be after all that loads.
function ECMVoicePackDriver::AutoBindPass(%this) {



	for (%i = 0; %i < %this.VoicePackCount; %i++) {
		
		if ((%this.VoicePacks[%i].DefaultKey !$= "") && (strpos($pref::TracerECM::InstalledVoicepacks, %this.VoicePacks[%i].ID) == -1)) {

			//Attempt to create an ECM Voicepack Entry point item at //#DEFAULTKEY specified location
			%keyConflict = false;
			
			%keySequence = %this.VoicePacks[%i].DefaultKey;
			%currentMenu = ECMenuBuild.RootMenu;

			echo("\C6AUTO-BIND FOR NEW VOICEPACK: \c1" @ %this.VoicePacks[%i].Name @ "\c6 - Key Sequence: \c1" @ %this.VoicePacks[%i].DefaultKey);

			while (%keySequence !$= "") {
				
				echo("\c6BEGIN PASS (CURRENT KEY-SEQUENCE: " @ %keySequence @ ")");

				//Get the location of next ':'
				%keyEnd = strpos(%keySequence, ":");
				
				//If none, last key entry
				if (%keyEnd == -1)
					%keyEnd = strlen(%keySequence);

				//Grab everything before the ":" (The Key Descriptor)
				%keydesc = getSubStr(%keySequence, 0, %keyEnd);
				
				echo("\c6 - KEY DESCRIPTOR: " @ %keydesc);

				//Remove the key that was grabbed from the string
				%keySequence = getSubStr(%keySequence, (%keyEnd + 1), (strlen(%keySequence) - (%keyEnd)));
				
				echo("\c6 - REMAINING KSEQ: " @ %keySequence);
				
				echo("\c6 - BEGIN MENU SCAN");
				//Check to see an item doesn't already exist at specified location, or if no path is available
				for (%a = 0; %a < %currentMenu.optionCount; %a++) {

					echo("\c6   - ENTRY #" @ %a);

					%menukey = getSubStr( %currentMenu.option[%a], 0, strpos(%currentMenu.option[%a], ":") );
					
					echo("\c6     - MENUKEY:" @ %menukey);
					
					//If there is stuff left in %keySequence, then we're checking to see if this item exists and is a menu
					if (%keySequence !$= "") {

						if (%menukey $= %keydesc) {
						
							if (%currentMenu.isMenu[%a]) {
								%currentMenu = %currentMenu.command[%a];
							} else {
								%keyConflict = true;
								%keySequence = "";
							}
							
							break;
							
						}

					//If there is no stuff in %keySequence, then we need to make sure NOTHING is already here
					} else {
						if (%keydesc $= %menukey)
							%keyConflict = true;
					}
				
				}
		
			}
			
			//If there is a conflict, then display a message, else, install a voicepack at this location...
			if (%keyConflict) {

				schedule(10000, 0, ECM_AutoBindErrMsg, %this.VoicePacks[%i].Name);
				
			} else {

				//STEP 1: Create a command object:
				%commandObj = new scriptObject(){};

				for (%x = 0; %x < $TracerECM::VoiceCount; %x++) {

					%commandObj.chatText[%x] = "";
					%commandObj.chatSound[%x] = "";
					%commandObj.nullChatText[%x] = "";
					%commandObj.nullChatSound[%x] = "";

				}
				%commandObj.isTeam = false;
				ECMenuBuild.CommandObjects.add(%commandObj);

				//STEP 2: Add the voicepack to the Voicepack entry point property on the command object
				%commandObj.startVoicePack = %this.VoicePacks[%i].ID;

				//STEP 3: Create item on menu
				//%key = getSubStr(%this.VoicePacks[%i].DefaultKey, 0, strpos(%this.VoicePacks[%i].DefaultKey, ":"));
				%text = %this.VoicePacks[%i].Name;
				%currentMenu.bindCmd(keyboard, %keydesc, "issueECMCmd(" @ %currentMenu @ "," @ %currentMenu.optionCount @ ");", "");
				%currentMenu.option[%currentMenu.optionCount] = %keydesc @ ": " @ %text;
				%currentMenu.isMenu[%currentMenu.optionCount] = 0;
				%currentMenu.command[%currentMenu.optionCount] = %commandObj;
				%currentMenu.optionCount++;

				//STEP 4: Save menu to file
				ECMenuBuild.SaveDataMenu();

	

			}
			//Mark First Time load flag for this voicepack
			$pref::TracerECM::InstalledVoicepacks = ($pref::TracerECM::InstalledVoicepacks @ %this.VoicePacks[%i].ID @ "\t");

		}
		
	}
	
}

function ECM_AutoBindErrMsg(%packname) {

	MessageBoxOK("AUTO-BIND CONFLICT", "Recently Installed Voicepack '" @ %packname @ "''s auto-bind key is to an existing key entry in your menu and was not added. You will have to add the voicepack manually.");

}

//Scan for voicepack files, and register them
ECMVoicePackDriver.ScanForVoicePacks();

//Install all registered voicepack files
ECMVoicePackDriver.InstallVoicePacks();




function ECM_Test1(%keySequence) {

	

}